// DiskSpace.cpp: implementation of the CDiskSpace class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DiskSpace.h"
#include <dos.h>
#include <direct.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDiskSpace::CDiskSpace()
{
	dwDrivesMask = 0;
	Refresh();
}

CDiskSpace::~CDiskSpace()
{
}

BOOL CDiskSpace::CheckIndex(int nIndex)
{
	if (!dwDrivesMask)
		Refresh();
	if (!dwDrivesMask)	return FALSE;
	if (nIndex<=0 || nIndex>=32)
		return FALSE;
	return TRUE;
}

BOOL CDiskSpace::CheckFormat(LPCTSTR lpszFormat)
{
	CString strFormat = lpszFormat;
	if (!strFormat.CompareNoCase(_T("mb")) ||
		!strFormat.CompareNoCase(_T("kb")) ||
		!strFormat.CompareNoCase(_T("bytes")))
		return TRUE;
	return FALSE;
}

BOOL CDiskSpace::GetDiskSpace(float &fOut,int nIndex,LPCTSTR lpszFormat)
{
	if (!CheckFormat(lpszFormat))
		return FALSE;
	if (!CheckIndex(nIndex))
		return FALSE;

	DWORD dwDrives = dwDrivesMask>>nIndex;
	if (!(dwDrives & 0x00000001))
		return FALSE; //no drive
	
	if (!GetData(fOut,nIndex,FALSE))
		return FALSE;
	return ModifyResult(fOut,lpszFormat);
}

BOOL CDiskSpace::GetData(float &fOut,int nIndex,BOOL bFreeSpace)
{
	struct _diskfree_t diskfree;
	memset(&diskfree,0,sizeof(diskfree));
	if (_getdiskfree(nIndex, &diskfree) == 0)
	{
		fOut = (float)diskfree.bytes_per_sector * (float)diskfree.sectors_per_cluster;
		fOut *= (bFreeSpace) ? (float)diskfree.avail_clusters : (float)diskfree.total_clusters;
		return TRUE;
	}
	return FALSE;
}

BOOL CDiskSpace::GetDiskSpace(float &fOut,LPCTSTR lpszDisk,LPCTSTR lpszFormat)
{
	if (!lpszDisk)
		return FALSE;
	TCHAR tch = (TCHAR) tolower((int)lpszDisk[0]); //letter
	if (tch<_T('a') || tch>_T('z'))
		return FALSE;

	int nIndex = tch-_T('a')+1;
	return GetDiskSpace(fOut,nIndex,lpszFormat);
}

BOOL CDiskSpace::GetDiskFreeSpace(float &fOut,int nIndex,LPCTSTR lpszFormat)
{
	if (!CheckFormat(lpszFormat))
		return FALSE;
	if (!CheckIndex(nIndex))
		return FALSE;

	DWORD dwDrives = dwDrivesMask>>nIndex;
	if (!(dwDrives & 0x00000001))
		return FALSE; //no drive
	
	if (!GetData(fOut,nIndex,TRUE))
		return FALSE;
	return ModifyResult(fOut,lpszFormat);
}

BOOL CDiskSpace::GetDiskFreeSpace(float &fOut,LPCTSTR lpszDisk,LPCTSTR lpszFormat)
{
	if (!lpszDisk)
		return FALSE;
	TCHAR tch = (TCHAR) tolower((int)lpszDisk[0]); //letter
	if (tch<_T('a') || tch>_T('z'))
		return FALSE;

	int nIndex = tch-_T('a')+1;
	return GetDiskFreeSpace(fOut,nIndex,lpszFormat);
}

BOOL CDiskSpace::ModifyResult(float &fOut,LPCTSTR lpszFormat)
{
	CString strFormat = lpszFormat;
	if (!strFormat.CompareNoCase(_T("mb"))) {
		fOut/=(float)1048576.0;
		return TRUE;
	}
	if (!strFormat.CompareNoCase(_T("kb"))) {
		fOut/=(float)1024.0;
		return TRUE;
	}
	if (!strFormat.CompareNoCase(_T("bytes")))
		return TRUE;
	return FALSE;
}


